@@ -2,6 +2,7 @@
    ** AJCD 4/9/90
  */
 
+#include <assert.h>
 #include <string.h>
 #include <stdio.h>
 
@@ -10,7 +11,7 @@
 #include "shhopt.h"
 #include "pnm.h"
 
-#define MAX_WIDTHHEIGHT INT_MAX-10
+#define MAX_WIDTHHEIGHT ((INT_MAX)-10)
     /* The maximum width or height value we can handle without risking
        arithmetic overflow
     */
@@ -309,14 +310,16 @@ computePadSizeBeforeMult(unsigned int   const unpaddedSize,
             *begPadP = begPadReq;
             *endPadP = MAX(sizeReq, unpaddedSize + begPadReq) -
                 (begPadReq + unpaddedSize);
-        } else if (endPadReq) {
+        } else if (endPadSpec) {
             *endPadP = endPadReq;
             *begPadP = MAX(sizeReq, unpaddedSize + endPadReq) -
                 (unpaddedSize + endPadReq);
         } else {
             if (sizeReq > unpaddedSize) {
+                assert(align <= 1.0 && align >= 0.0);
                 *begPadP = ROUNDU((sizeReq - unpaddedSize) * align);
                 *endPadP = sizeReq - unpaddedSize - *begPadP;
+                assert(*begPadP + unpaddedSize + *endPadP == sizeReq);
             } else {
                 *begPadP = 0;
                 *endPadP = 0;
